@@ -245,23 +245,22 @@ static char * deptypeFormat(rpmtd td, char * formatPrefix)
 	    argvAdd(&sdeps, "postun");
 	if (item & RPMSENSE_SCRIPT_VERIFY)
 	    argvAdd(&sdeps, "verify");
+	if (item & RPMSENSE_INTERP)
+	    argvAdd(&sdeps, "interp");
+	if (item & RPMSENSE_RPMLIB)
+	    argvAdd(&sdeps, "rpmlib");
+	if ((item & RPMSENSE_FIND_REQUIRES) || (item & RPMSENSE_FIND_PROVIDES))
+	    argvAdd(&sdeps, "auto");
+	if (item & RPMSENSE_PREREQ)
+	    argvAdd(&sdeps, "prereq");
 
 	if (sdeps) {
 	    val = argvJoin(sdeps, ",");
-	    argvFree(sdeps);
 	} else {
-	    if (item & RPMSENSE_RPMLIB)
-		val = xstrdup("rpmlib");
-	    else if (item & RPMSENSE_INTERP)
-		val = xstrdup("interp");
-	    else if ((item & RPMSENSE_FIND_REQUIRES) || 
-		     (item & RPMSENSE_FIND_PROVIDES))
-		val = xstrdup("auto");
-	    else if (item & RPMSENSE_PREREQ)
-		val = xstrdup("prereq");
-	    else
-		val = xstrdup("manual");
+	    val = xstrdup("manual");
 	}
+
+	argvFree(sdeps);
     }
     return val;
 }
@@ -515,33 +514,11 @@ static char * pgpsigFormat(rpmtd td, char * formatPrefix)
 	    nb += 100;
 	    val = t = xrealloc(val, nb + 1);
 
-	    switch (sigp->pubkey_algo) {
-	    case PGPPUBKEYALGO_DSA:
-		t = stpcpy(t, "DSA");
-		break;
-	    case PGPPUBKEYALGO_RSA:
-		t = stpcpy(t, "RSA");
-		break;
-	    default:
-		(void) snprintf(t, nb - (t - val), "%d", sigp->pubkey_algo);
-		t += strlen(t);
-		break;
-	    }
+	    t = stpcpy(t, pgpValString(PGPVAL_PUBKEYALGO, sigp->pubkey_algo));
 	    if (t + 5 >= val + nb)
 		goto again;
 	    *t++ = '/';
-	    switch (sigp->hash_algo) {
-	    case PGPHASHALGO_MD5:
-		t = stpcpy(t, "MD5");
-		break;
-	    case PGPHASHALGO_SHA1:
-		t = stpcpy(t, "SHA1");
-		break;
-	    default:
-		(void) snprintf(t, nb - (t - val), "%d", sigp->hash_algo);
-		t += strlen(t);
-		break;
-	    }
+	    t = stpcpy(t, pgpValString(PGPVAL_HASHALGO, sigp->hash_algo));
 	    if (t + strlen (", ") + 1 >= val + nb)
 		goto again;
 
