@@ -101,7 +101,6 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
     rpmfi fi = NULL;
     rpmfiFlags fiflags =  (RPMFI_NOHEADER | RPMFI_FLAGS_QUERY);
     int rc = 0;		/* XXX FIXME: need real return code */
-    int i;
 
     if (qva->qva_queryFormat != NULL) {
 	const char *errstr;
@@ -128,7 +127,7 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
     }
 
     fi = rpmfiInit(fi, 0);
-    while ((i = rpmfiNext(fi)) >= 0) {
+    while (rpmfiNext(fi) >= 0) {
 	rpmfileAttrs fflags = rpmfiFFlags(fi);
 	rpm_mode_t fmode = rpmfiFMode(fi);
 	rpm_rdev_t frdev = rpmfiFRdev(fi);
@@ -139,7 +138,6 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 	const char *fuser = rpmfiFUser(fi);
 	const char *fgroup = rpmfiFGroup(fi);
 	const char *flink = rpmfiFLink(fi);
-	uint32_t fnlink = rpmfiFNlink(fi);
 	char *buf = NULL;
 
 	/* If querying only docs, skip non-doc files. */
@@ -210,6 +208,7 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 	    rpmlog(RPMLOG_NOTICE, "%s%s\n", buf ? buf : "", fn);
 	}
 	else {
+	    uint32_t fnlink = rpmfiFNlink(fi);
 
 	    /* XXX Adjust directory link count and size for display output. */
 	    if (S_ISDIR(fmode)) {
@@ -280,10 +279,8 @@ static int rpmgiShowMatches(QVA_t qva, rpmts ts)
 	    continue;
 	if ((rc = qva->qva_showPackage(qva, ts, h)) != 0)
 	    ec = rc;
-	if (qva->qva_source == RPMQV_DBOFFSET)
-	    break;
     }
-    return rpmgiNumErrors(gi);
+    return ec + rpmgiNumErrors(gi);
 }
 
 int rpmcliShowMatches(QVA_t qva, rpmts ts)
@@ -296,14 +293,11 @@ int rpmcliShowMatches(QVA_t qva, rpmts ts)
 	rpmdbCheckSignals();
 	if ((rc = qva->qva_showPackage(qva, ts, h)) != 0)
 	    ec = rc;
-	if (qva->qva_source == RPMQV_DBOFFSET)
-	    break;
     }
     qva->qva_mi = rpmdbFreeIterator(qva->qva_mi);
     return ec;
 }
 
-/* LCL: segfault (realpath annotation?) */
 int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
 {
     int res = 0;
@@ -318,17 +312,8 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
 
     switch (qva->qva_source) {
     case RPMQV_RPM:
-	res = rpmgiShowMatches(qva, ts);
-	break;
-
     case RPMQV_ALL:
-	res = rpmgiShowMatches(qva, ts);
-	break;
-
     case RPMQV_HDLIST:
-	res = rpmgiShowMatches(qva, ts);
-	break;
-
     case RPMQV_FTSWALK:
 	res = rpmgiShowMatches(qva, ts);
 	break;
@@ -427,22 +412,10 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
     }	break;
 
     case RPMQV_TID:
-    {	int mybase = 10;
-	const char * myarg = arg;
-	char * end = NULL;
-	unsigned long iid;
+    {	char * end = NULL;
+	rpm_tid_t iid = strtoul(arg, &end, 0);
 
-	/* XXX should be in strtoul */
-	if (*myarg == '0') {
-	    myarg++;
-	    mybase = 8;
-	    if (*myarg == 'x') {
-		myarg++;
-		mybase = 16;
-	    }
-	}
-	iid = strtoul(myarg, &end, mybase);
-	if ((*end) || (end == arg) || (iid == ULONG_MAX)) {
+	if ((*end) || (end == arg) || (iid == UINT_MAX)) {
 	    rpmlog(RPMLOG_ERR, _("malformed %s: %s\n"), "tid", arg);
 	    return 1;
 	}
@@ -513,31 +486,19 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
     }	break;
 
     case RPMQV_DBOFFSET:
-    {	int mybase = 10;
-	const char * myarg = arg;
-	char * end = NULL;
-	unsigned long recOffset;
+    {	char * end = NULL;
+	unsigned int recOffset = strtoul(arg, &end, 0);
 
-	/* XXX should be in strtoul */
-	if (*myarg == '0') {
-	    myarg++;
-	    mybase = 8;
-	    if (*myarg == 'x') {
-		myarg++;
-		mybase = 16;
-	    }
-	}
-	recOffset = strtoul(myarg, &end, mybase);
-	if ((*end) || (end == arg) || (recOffset == ULONG_MAX)) {
+	if ((*end) || (end == arg) || (recOffset == UINT_MAX)) {
 	    rpmlog(RPMLOG_ERR, _("invalid package number: %s\n"), arg);
 	    return 1;
 	}
-	rpmlog(RPMLOG_DEBUG, "package record number: %lu\n", recOffset);
+	rpmlog(RPMLOG_DEBUG, "package record number: %u\n", recOffset);
 	/* RPMDBI_PACKAGES */
 	qva->qva_mi = rpmtsInitIterator(ts, RPMDBI_PACKAGES, &recOffset, sizeof(recOffset));
 	if (qva->qva_mi == NULL) {
 	    rpmlog(RPMLOG_ERR,
-		_("record %lu could not be read\n"), recOffset);
+		_("record %u could not be read\n"), recOffset);
 	    res = 1;
 	} else
 	    res = rpmcliShowMatches(qva, ts);
@@ -567,69 +528,46 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
     return res;
 }
 
-int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_const_t argv)
+static int rpmcliArgIterHelper(rpmts ts, QVA_t qva, rpmTag tag, ARGV_const_t argv, rpmgiFlags gFlgs)
 {
     rpmRC rpmrc = RPMRC_NOTFOUND;
     int ec = 0;
 
+    qva->qva_gi = rpmgiNew(ts, tag, NULL, 0);
+    qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, gFlgs);
+    
+    if (qva->qva_gi != NULL && (rpmgiGetFlags(qva->qva_gi) & RPMGI_TSADD))	/* Load the ts with headers. */
+        while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
+	    {};
+    if (rpmrc != RPMRC_NOTFOUND) {
+        qva->qva_gi = rpmgiFree(qva->qva_gi);
+	return 1;	/* XXX should be no. of failures. */
+    }
+    /* FIX: argv can be NULL, cast to pass argv array */
+    ec = rpmQueryVerify(qva, ts, (tag == RPMDBI_PACKAGES)? (const char *) argv : NULL);
+    rpmtsEmpty(ts);
+    qva->qva_gi = rpmgiFree(qva->qva_gi);
+    return ec;
+}
+
+int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_const_t argv)
+{
+    int ec = 0;
+
     switch (qva->qva_source) {
     case RPMQV_ALL:
-	qva->qva_gi = rpmgiNew(ts, RPMDBI_PACKAGES, NULL, 0);
-	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, RPMGI_NONE);
-
-	if (qva->qva_gi != NULL && (rpmgiGetFlags(qva->qva_gi) & RPMGI_TSADD))	/* Load the ts with headers. */
-	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
-	    {};
-	if (rpmrc != RPMRC_NOTFOUND)
-	    return 1;	/* XXX should be no. of failures. */
-	
-	/* FIX: argv can be NULL, cast to pass argv array */
-	ec = rpmQueryVerify(qva, ts, (const char *) argv);
-	rpmtsEmpty(ts);
+	ec = rpmcliArgIterHelper(ts, qva, RPMDBI_PACKAGES, argv, RPMGI_NONE);
 	break;
     case RPMQV_RPM:
-	qva->qva_gi = rpmgiNew(ts, RPMDBI_ARGLIST, NULL, 0);
-	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
-
-	if (qva->qva_gi != NULL && (rpmgiGetFlags(qva->qva_gi) & RPMGI_TSADD))	/* Load the ts with headers. */
-	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
-	    {};
-	if (rpmrc != RPMRC_NOTFOUND)
-	    return 1;	/* XXX should be no. of failures. */
-	
-	/* FIX: argv can be NULL, cast to pass argv array */
-	ec = rpmQueryVerify(qva, ts, NULL);
-	rpmtsEmpty(ts);
+	ec = rpmcliArgIterHelper(ts, qva, RPMDBI_ARGLIST, argv, giFlags);
 	break;
     case RPMQV_HDLIST:
-	qva->qva_gi = rpmgiNew(ts, RPMDBI_HDLIST, NULL, 0);
-	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
-
-	if (qva->qva_gi != NULL && (rpmgiGetFlags(qva->qva_gi) & RPMGI_TSADD))	/* Load the ts with headers. */
-	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
-	    {};
-	if (rpmrc != RPMRC_NOTFOUND)
-	    return 1;	/* XXX should be no. of failures. */
-	
-	/* FIX: argv can be NULL, cast to pass argv array */
-	ec = rpmQueryVerify(qva, ts, NULL);
-	rpmtsEmpty(ts);
+	ec = rpmcliArgIterHelper(ts, qva, RPMDBI_HDLIST, argv, giFlags);
 	break;
     case RPMQV_FTSWALK:
 	if (ftsOpts == 0)
 	    ftsOpts = (RPMGI_COMFOLLOW | RPMGI_LOGICAL | RPMGI_NOSTAT);
-	qva->qva_gi = rpmgiNew(ts, RPMDBI_FTSWALK, NULL, 0);
-	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts, giFlags);
-
-	if (qva->qva_gi != NULL && (rpmgiGetFlags(qva->qva_gi) & RPMGI_TSADD))	/* Load the ts with headers. */
-	while ((rpmrc = rpmgiNext(qva->qva_gi)) == RPMRC_OK)
-	    {};
-	if (rpmrc != RPMRC_NOTFOUND)
-	    return 1;	/* XXX should be no. of failures. */
-	
-	/* FIX: argv can be NULL, cast to pass argv array */
-	ec = rpmQueryVerify(qva, ts, NULL);
-	rpmtsEmpty(ts);
+	ec = rpmcliArgIterHelper(ts, qva, RPMDBI_FTSWALK, argv, giFlags);
 	break;
     default:
 	qva->qva_gi = rpmgiNew(ts, RPMDBI_ARGLIST, NULL, 0);
@@ -639,11 +577,10 @@ int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_const_t argv)
 	    ec += rpmQueryVerify(qva, ts, rpmgiHdrPath(qva->qva_gi));
 	    rpmtsEmpty(ts);
 	}
+	qva->qva_gi = rpmgiFree(qva->qva_gi);
 	break;
     }
 
-    qva->qva_gi = rpmgiFree(qva->qva_gi);
-
     return ec;
 }
 
@@ -660,7 +597,7 @@ int rpmcliQuery(rpmts ts, QVA_t qva, char * const * argv)
 	char * fmt = rpmExpand("%{?_query_all_fmt}\n", NULL);
 	if (fmt == NULL || strlen(fmt) <= 1) {
 	    fmt = _free(fmt);
-	    fmt = xstrdup("%{name}-%{version}-%{release}.%{arch}\n");
+	    fmt = xstrdup("%{nvra}\n");
 	}
 	qva->qva_queryFormat = fmt;
     }
