@@ -221,12 +221,10 @@ static int rpmReSign(rpmts ts, QVA_t qva, ARGV_const_t argv)
 
 	    oh = headerCopyLoad(utd.data);
 	    hi = headerInitIterator(oh);
-	    for (hi = headerInitIterator(oh);
-		headerNext(hi, &copytd);
-		rpmtdFreeData(&copytd))
-	    {
+	    while (headerNext(hi, &copytd)) {
 		if (copytd.data)
 		    xx = headerPut(nh, &copytd, HEADERPUT_DEFAULT);
+		rpmtdFreeData(&copytd);
 	    }
 	    hi = headerFreeIterator(hi);
 	    oh = headerFree(oh);
@@ -593,12 +591,9 @@ static void formatResult(rpmSigTag sigtag, rpmRC sigres, const char *result,
 	if (havekey && (sigres == RPMRC_NOKEY || sigres == RPMRC_NOTTRUSTED)) {
 	    const char *tempKey = strstr(result, "ey ID");
 	    if (tempKey) {
-		char *keyid = strndup(tempKey + 6, 8);
-	    	char *idprob = NULL;
-		rasprintf(&idprob, " %s#%s", signame, keyid);
-		rstrcat(keyprob, idprob);
-		free(keyid);
-		free(idprob);
+		char keyid[sizeof(pgpKeyID_t) + 1];
+		rstrlcpy(keyid, tempKey + 6, sizeof(keyid));
+		rstrscat(keyprob, " ", signame, "#", keyid, NULL);
 	    }
 	}
 	rasprintf(&msg, (*keyprob ? "(%s) " : "%s "), signame);
