@@ -26,8 +26,6 @@ struct rpmts_s {
     rpmprobFilterFlags ignoreSet;
 				/*!< Bits to filter current problems. */
 
-    unsigned int filesystemCount;	/*!< No. of mounted filesystems. */
-    const char ** filesystems;	/*!< Mounted filesystem names. */
     rpmDiskSpaceInfo dsi;	/*!< Per filesystem disk/inode usage. */
 
     rpmdb rdb;			/*!< Install database handle. */
