@@ -10,7 +10,7 @@
 #endif
 #define	iseol(_c)	((_c) == '\n' || (_c) == '\r')
 
-#define	STREQ(_t, _f, _fn)	((_fn) == (sizeof(_t)-1) && !strncmp((_t), (_f), (_fn)))
+#define	STREQ(_t, _f, _fn)	((_fn) == (sizeof(_t)-1) && rstreqn((_t), (_f), (_fn)))
 
 #define MACROBUFSIZ (BUFSIZ * 2)
 
@@ -780,7 +780,7 @@ static const char *
 grabArgs(MacroBuf mb, const rpmMacroEntry me, const char * se,
 		const char * lastc)
 {
-    const char *opts, *o;
+    const char *opts;
     char *args = NULL;
     ARGV_t argv = NULL;
     int argc = 0;
@@ -834,7 +834,7 @@ grabArgs(MacroBuf mb, const rpmMacroEntry me, const char * se,
     while((c = getopt(argc, argv, opts)) != -1)
     {
 	char *name = NULL, *body = NULL;
-	if (c == '?' || (o = strchr(opts, c)) == NULL) {
+	if (c == '?' || strchr(opts, c) == NULL) {
 	    rpmlog(RPMLOG_ERR, _("Unknown option %c in %s(%s)\n"),
 			(char)c, me->name, opts);
 	    goto exit;
@@ -979,8 +979,6 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 	    sprintf(be, "%%__unzip %s", b);
 	    break;
         case COMPRESSED_LZMA:
-            sprintf(be, "%%__lzma -dc %s", b);
-            break;
         case COMPRESSED_XZ:
             sprintf(be, "%%__xz -dc %s", b);
             break;
