@@ -202,14 +202,7 @@ static void pgpPrtStr(const char *pre, const char *s)
     fprintf(stderr, " %s", s);
 }
 
-/** \ingroup rpmpgp
- * Return string representation of am OpenPGP value.
- * @param vs		table of (string,value) pairs
- * @param val		byte value to lookup
- * @return		string value of byte
- */
-static inline
-const char * pgpValStr(pgpValTbl vs, uint8_t val)
+static const char * pgpValStr(pgpValTbl vs, uint8_t val)
 {
     do {
 	if (vs->val == val)
@@ -218,6 +211,31 @@ const char * pgpValStr(pgpValTbl vs, uint8_t val)
     return vs->str;
 }
 
+static pgpValTbl pgpValTable(pgpValType type)
+{
+    switch (type) {
+    case PGPVAL_TAG:		return pgpTagTbl;
+    case PGPVAL_ARMORBLOCK:	return pgpArmorTbl;
+    case PGPVAL_ARMORKEY:	return pgpArmorKeyTbl;
+    case PGPVAL_SIGTYPE:	return pgpSigTypeTbl;
+    case PGPVAL_SUBTYPE:	return pgpSubTypeTbl;
+    case PGPVAL_PUBKEYALGO:	return pgpPubkeyTbl;
+    case PGPVAL_SYMKEYALGO:	return pgpSymkeyTbl;
+    case PGPVAL_COMPRESSALGO:	return pgpCompressionTbl;
+    case PGPVAL_HASHALGO: 	return pgpHashTbl;
+    case PGPVAL_SERVERPREFS:	return pgpKeyServerPrefsTbl;
+    default:
+	break;
+    }
+    return NULL;
+}
+
+const char * pgpValString(pgpValType type, uint8_t val)
+{
+    pgpValTbl tbl = pgpValTable(type);
+    return (tbl != NULL) ? pgpValStr(tbl, val) : NULL;
+}
+
 static void pgpPrtHex(const char *pre, const uint8_t *p, size_t plen)
 {
     char *hex = NULL;
@@ -1448,10 +1466,13 @@ static pgpArmor decodePkts(uint8_t *b, uint8_t **pkt, size_t *pktlen)
 	    }
 	    if (rc != PGPARMOR_PUBKEY)	/* XXX ASCII Pubkeys only, please. */
 		continue;
-	    armortype = t;
 
-	    t = te - (sizeof("-----\n")-1);
-	    if (!TOKEQ(t, "-----\n"))
+	    armortype = pgpValStr(pgpArmorTbl, rc);
+	    t += strlen(armortype);
+	    if (!TOKEQ(t, "-----"))
+		continue;
+	    t += sizeof("-----")-1;
+	    if (*t != '\n' && *t != '\r')
 		continue;
 	    *t = '\0';
 	    pstate++;
@@ -1461,7 +1482,7 @@ static pgpArmor decodePkts(uint8_t *b, uint8_t **pkt, size_t *pktlen)
 	    rc = pgpValTok(pgpArmorKeyTbl, t, te);
 	    if (rc >= 0)
 		continue;
-	    if (*t != '\n') {
+	    if (*t != '\n' && *t != '\r') {
 		pstate = 0;
 		continue;
 	    }
