@@ -127,6 +127,7 @@ static struct poptOption optionsTable[] = {
 long _stksize = 64 * 1024L;
 #endif
 
+RPM_GNUC_NORETURN
 static void argerror(const char * desc)
 {
     fprintf(stderr, _("%s: %s\n"), __progname, desc);
@@ -210,11 +211,11 @@ int main(int argc, char *argv[])
 
     /* Set the major mode based on argv[0] */
 #ifdef	IAM_RPMBT
-    if (!strcmp(__progname, "rpmbuild"))	bigMode = MODE_BUILD;
+    if (rstreq(__progname, "rpmbuild"))	bigMode = MODE_BUILD;
 #endif
 #ifdef	IAM_RPMQV
-    if (!strcmp(__progname, "rpmquery"))	bigMode = MODE_QUERY;
-    if (!strcmp(__progname, "rpmverify"))	bigMode = MODE_VERIFY;
+    if (rstreq(__progname, "rpmquery"))	bigMode = MODE_QUERY;
+    if (rstreq(__progname, "rpmverify")) bigMode = MODE_VERIFY;
 #endif
 
 #if defined(IAM_RPMQV)
@@ -538,14 +539,6 @@ int main(int argc, char *argv[])
 		  case 0:
 		    break;
 		  case RPMSIGTAG_PGP:
-#ifdef	DYING	/* XXX gpg can now be used for RSA signatures. */
-		    if ((sigTag == RPMSIGTAG_PGP || sigTag == RPMSIGTAG_PGP5) &&
-		        !rpmDetectPGPVersion(NULL)) {
-		        fprintf(stderr, _("pgp not found: "));
-			ec = EXIT_FAILURE;
-			goto exit;
-		    }
-#endif
 		  case RPMSIGTAG_GPG:
 		  case RPMSIGTAG_DSA:
 		  case RPMSIGTAG_RSA:
